module doodle.fig.select_tool;

public {
    import doodle.dia.tool;
}

final class SelectTool : Tool {
    this() {
        super("Select");
    }

    override bool handleButtonPress(scope IViewport viewport, in ButtonEvent event) {
        if (event.buttonName == ButtonName.LEFT) {
            _active = true;
            _anchorPoint = _currentPoint = event.screenPoint;
            viewport.setCursor(Cursor.HAND);
            return true;
        }
        else {
            return false;
        }
    }

    override bool handleButtonRelease(scope IViewport viewport, in ButtonEvent event) {
        if (event.buttonName == ButtonName.LEFT && _active) {
            _active = false;
            viewport.damageScreen(growCentre(Rectangle(_anchorPoint, _currentPoint), LINE_WIDTH));
            viewport.setCursor(Cursor.DEFAULT);
            return true;
        }
        else {
            return false;
        }
    }

    override bool handleMotion(scope IViewport viewport, in MotionEvent event) {
        if (_active) {
            viewport.damageScreen(growCentre(Rectangle(_anchorPoint, _currentPoint), LINE_WIDTH));
            _currentPoint = event.screenPoint;
            viewport.damageScreen(growCentre(Rectangle(_anchorPoint, _currentPoint), LINE_WIDTH));
        }

        return false;
    }

    override void draw(in Rectangle screenDamage, scope Drawable screenDrawable) const {
        if (_active) {
            screenDrawable.pushState; {
                screenDrawable.setLineStyle(Drawable.LineStyle.DASHED);
                screenDrawable.setLineWidth(LINE_WIDTH);
                screenDrawable.setColor(Color(0.0, 0.0, 0.5, 1.0));
                screenDrawable.drawRectangle(Rectangle(_currentPoint, _anchorPoint), false);
            } screenDrawable.popState;
        }
    }

    private {
        bool _active;
        Point _currentPoint;
        Point _anchorPoint;
        static immutable double LINE_WIDTH = 1.0;
    }
}
